<?php

function get_tick($f)
{
  return intval((1500.0+$f*900.0)/20000.0*4096.0);
}

function tick2angle($tick)
{
  $f=($tick*20000.0/4096.0-1500.0)/900;
  $ret=$f*M_PI/2.0;
  return $ret;
}

function move_servo($h,$tick)
{
  fwrite($h,"0 ".$tick);
}

function distance_scan_on()
{
  global $fgp;

  exec("gpio-event 3:b:0");
  file_put_contents("/dev/v2r_gpio","set gpio 3 input");
  file_put_contents("/dev/v2r_pins","set con 44 pwm1");
  file_put_contents("/dev/v2r_pins","set pwm 1 100 400000");
  
  $fgp = fopen("/dev/gpio-event", "r");
}

function distance_scan_off()
{
  global $fgp;

  file_put_contents("/dev/v2r_pins","set con 44 input");
  file_put_contents("/sys/bus/i2c/devices/1-0070/ch6","0 0");
  file_put_contents("/sys/bus/i2c/devices/1-0070/ch7","0 0");

  fclose($fgp);
}

function get_one_shot($f)
{
  $s2=fgets($f);

  $t0=0;

  while(true)
  {
    $s=fgets($f);
    if(sscanf($s,'%s %s %s',$cn,$rising,$stamp)!=3)
      continue;

    if($cn!=3)
      continue;

    if($rising=="R")
    {
       $t0=$stamp;
    }
    else if($t0!=0)
    {
       $t0=$stamp-$t0;
       break;
    }
  }

  $t0=$t0*1000000.0/58.0;
  return $t0;
}

function get_distance()
{
  global $fgp;
  
  $v=0.0;
  $c=0;
  for($i=0;$i<100;$i++)
  {
    $vi=get_one_shot($fgp);
    if($vi<30 || $vi>600)
     continue;
    $v+=$vi;
    $c++;
  }

  if($c==0)$v=1000.0;
  else $v/=$c;

  return $v;
}

function get_idx($x,$y)
{
  global $minx,$miny,$maxx,$maxy;
  global $img_width,$img_height,$img_data,$step;

  return $img_width*intval(($y-$miny)/$step)+intval(($x-$minx)/$step);
}

function scan_position($x,$y)
{
  global $hx,$hy;
  global $minx,$miny,$maxx,$maxy;
  global $img_width,$img_height,$img_data,$step;

  move_servo($hx,$x);
  move_servo($hy,$y);
//  usleep(100000);

  $d=get_distance();
  $idx=get_idx($x,$y);
  $img_data[$idx]=$d;
	
  echo "$idx: ($x,$y) of ($maxx,$maxy): d=$d\n";
}

function scan()
{
  global $hx,$hy;
  global $minx,$miny,$maxx,$maxy;
  global $img_width,$img_height,$img_data,$step;

  $img_data=array();

	move_servo($hx,$minx);
	move_servo($hy,$miny);

	usleep(1000000);

	$x=$minx;

	for($y=$miny;$y<$maxy;$y+=$step)
	{
	  if($x<=$minx)
	  {
		  for($x=$minx;$x<$maxx;$x+=$step)
		  {
			scan_position($x,$y);
		  }
	  }
	  else
	  {
		  for($x=$minx+($img_width-1)*$step;$x>=$minx;$x-=$step)
		  {
			scan_position($x,$y);
		  }
	  }

    }
}

function generate_vertexes()
{
  global $minx,$miny,$maxx,$maxy;
  global $img_width,$img_height,$img_data,$step;

  $str="";
  
  for($y=$miny;$y<$maxy;$y+=$step)
  {
	  for($x=$minx;$x<$maxx;$x+=$step)
	  {
	    $idx=get_idx($x,$y);
//		echo "$idx $x $minx $y $miny $img_width\n";
	    $d=$img_data[$idx];
		$ay=tick2angle($y);
		$ax=tick2angle($x);
		
		$cz=$d*sin($ay);
		$xy=$d*cos($ay);
		
		$cx=$xy*cos($ax);
		$cy=$xy*sin($ax);
		
		$str=$str."$cx $cy $cz\n";
	  }
	  
  }
  
  return $str;
}

function generate_triangle($a,$b,$c)
{
  global $img_data;
  global $triangles_count;
  global $distance_limit;
  
  $da=$img_data[$a];
  $db=$img_data[$b];
  $dc=$img_data[$c];
  
  if(
    (abs($da-$db)<$distance_limit) &&
    (abs($da-$dc)<$distance_limit) &&
    (abs($db-$dc)<$distance_limit))
  {
    $triangles_count++;
	return "3 $a $b $c\n";
  }
  
  return "";
}

function generate_triangles()
{
  global $img_width,$img_height,$img_data;
  global $triangles_count;
  
  $triangles_count=0;

  $str="";

  for($y=0;$y<$img_height-1;$y++)
  {
	for($x=0;$x<$img_width-1;$x++)
	{
		$a=$y*$img_width+$x;
		$b=$a+1;
		$c=$a+$img_width;
		$d=$b+$img_width;
		
		$str=$str.generate_triangle($a,$b,$c);
		$str=$str.generate_triangle($c,$b,$d);
	}
  }
  
  return $str;
}

function output_data()
{
  global $img_width,$img_height,$img_data;
  global $triangles_count;

  $f=fopen("/var/www/scan.off","w");
  $vertex_count=$img_width*$img_height;
  $v=generate_vertexes();
  $t=generate_triangles();
  
  fwrite($f,"OFF\n");
  fwrite($f,"$vertex_count $triangles_count 0\n");
  fwrite($f,"\n");
  fwrite($f,$v);
  fwrite($f,$t);
}

function get_dist_data($x,$y)
{
  global $img_width,$img_height,$img_data;
  return $img_data[$y*$img_width+$x];
}

function smooth($dx,$dy)
{
  global $img_width,$img_height,$img_data;
  global $distance_limit;

  $na=$img_data;
  
  for($y=$dy;$y<$img_height-$dy;$y++)
  for($x=$dx;$x<$img_width-$dx;$x++)
  {
	$c=1;
    $p=get_dist_data($x,$y);
	$acc=$p;

	$l=get_dist_data($x-$dx,$y-$dy);
	$r=get_dist_data($x+$dx,$y+$dy);

	if(abs($p-$l)<$distance_limit)
	{
	  $acc+=$l;
	  $c++;
	}

	if(abs($p-$r)<$distance_limit)
	{
	  $acc+=$r;
	  $c++;
	}
	
	$acc/=$c;
	$na[$y*$img_width+$x]=$acc;
  }
  
  $img_data=$na;
}

$hx = fopen("/sys/bus/i2c/devices/1-0070/ch7", "w");
$hy = fopen("/sys/bus/i2c/devices/1-0070/ch6", "w");

$minx=get_tick(-0.3);
$maxx=get_tick(0.3);
$miny=get_tick(-0.3);
$maxy=get_tick(0.1);
$distance_limit=100;
$step=10;
echo "($minx $miny;$maxx $maxy)\n";

$img_width=intval(($maxx-$minx+$step-1)/$step);
$img_height=intval(($maxy-$miny+$step-1)/$step);
$img_data=array();

distance_scan_on();

scan();
distance_scan_off();
//smooth(1,0);
//smooth(0,1);
output_data();

?>
